/*-----------------------------------------------------------------------

File  : source_handler.c

Author: Stephan Schulz

Contents
 
  File local variables and access functions realising the source
  handler. 

  Copyright 2002 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Thu Sep 19 23:55:55 CEST 2002
    New

-----------------------------------------------------------------------*/

#include "source_handler.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

static int current_char;
static int current_line;
static int current_column;
static char* input_name = NULL;
static FILE* input_stream;

/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: InitSourceHandler()
//
//   Initialize the source handler: Open the file, read the first
//   character, set all variables to rational values.
//
// Global Variables: Only file local ones (and all of those)
//
// Side Effects    : See above
//
/----------------------------------------------------------------------*/

void InitSourceHandler(char* filename)
{
   assert(!input_name&& "Source handler already initialized!");

   
   input_stream = fopen(filename, "r");
   if(!input_stream)
   {
      perror(ProgName);
      exit(EXIT_FAILURE);
   }
   current_line = 1;
   current_column = 1;
   input_name = SecureStrdup(filename);
   current_char = fgetc(input_stream);
}

/*-----------------------------------------------------------------------
//
// Function: ExitSourceHandler()
//
//   Exit the source handler, closing the input file and freeing the
//   memory. 
//
// Global Variables: As above
//
// Side Effects    : See above
//
/----------------------------------------------------------------------*/

void ExitSourceHandler()
{
   assert(input_name&&"Source handle not in use!");
   
   if(fclose(input_stream)!=0)
   {
      perror(ProgName);
      exit(EXIT_FAILURE);
   }
   free(input_name);
   input_name = NULL;   
   current_char = EOF;
   current_line = -1;
   current_column = -11;
}


/*-----------------------------------------------------------------------
//
// Function: NextChar()
//
//   Read the next character and return it.
//
// Global Variables: current_char,current_line,current_column
//
// Side Effects    : Changes the global variables
//
/----------------------------------------------------------------------*/

int  NextChar()
{
   if(current_char == '\n')
   {
      current_line++;
      current_column=1;
   }
   else
   {
      current_column++;
   }
   current_char = fgetc(input_stream);

   return current_char;
}


/*-----------------------------------------------------------------------
//
// Function: CurrentChar(), CurrentLine(), CurrentColumn()
//
//   Return the values of the corresponding variables.
//
// Global Variables: current_char,current_line,current_column
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

int  CurrentChar()
{
   return current_char;
}
int  CurrentLine()
{
   return current_line;
}
int  CurrentColumn()
{
   return current_column;
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/








